#!/bin/sh

/**
 * Copyright (c)2022-2025 National Fundamental Software of China Co., Ltd.
 *
 * You should have find a copy of NFSChina License. lf not, please contact us
 * by email <os_support@nfschina.com>
 *
**/

set -e
export OS_PROBER_DISABLE_DEBUG=1
. /usr/share/os-prober/common.sh

newns "$@"
require_tmpdir

partitions () {
	os_name="$(uname -s)"
	# Exclude partitions that have whole_disk sysfs attribute set.
	if [ -d /sys/block ]; then
		# Exclude partitions on physical disks that are part of a Serial ATA RAID disk.
		for part in /sys/block/*/*[0-9]; do
			if [ -f "$part/start" ] && \
			   [ ! -f "$part/whole_disk" ]; then
				name="$(echo "${part##*/}" | sed 's,[!.],/,g')"
				if [ -e "/dev/$name" ]; then
					echo "/dev/$name"
				fi
			fi
		done
		# Not support Serial ATA RAID devices
	elif [ "$os_name" = Linux ]; then
		echo "Cannot find list of partitions! " >&2
		exit 1
	else
		# Don't support any other os
		exit 0
	fi
	# Got the btrfs partitions and disks
	blkid | grep 'TYPE="btrfs"' | cut -d ':' -f 1
}

parse_proc_swaps () {
	while read line; do
		set -f
		set -- $line
		set +f
		echo "$(mapdevfs $1) swap"
	done
}

# Needed for idempotency
rm -f /var/lib/os-prober/labels

#for prog in /usr/lib/os-probes/init/*; do
#	if [ -x "$prog" ] && [ -f "$prog" ]; then
#		"$prog" || true
#	fi
#done

# If partitions in RAID, skip the detect
mount | grep "^/dev/" | parse_proc_mounts >"$OS_PROBER_TMP/mounted-map" || true
: >"$OS_PROBER_TMP/swaps-map"
if [ -f /proc/swaps ]; then
	grep "^/dev/" /proc/swaps | parse_proc_swaps >"$OS_PROBER_TMP/swaps-map" || true
fi
# Not support raided-map and btrfs-vols

#echo $(partitions)
for partition in $(partitions); do
	if ! mapped="$(mapdevfs "$partition")"; then
		log "Device '$partition' does not exist; skipping"
		continue
	fi

	# Skip partitions used as active swap
	if grep -q "^$mapped " "$OS_PROBER_TMP/swaps-map" ; then
		debug "$partition: is active swap"
		continue
	fi

	if ! grep -q "^$mapped " "$OS_PROBER_TMP/mounted-map" ; then
		for test in /usr/lib/os-probes/*; do
			if [ -f "$test" ] && [ -x "$test" ]; then
				debug "running $test on $partition"
				if "$test" "$partition"; then
					debug "os detected by $test"
			   		break
				fi
			fi
		done
	else
		mpoint=$(grep "^$mapped " "$OS_PROBER_TMP/mounted-map" | head -n1 | cut -d " " -f 2)
		mpoint="$(unescape_mount "$mpoint")"
		if [ "$mpoint" != "/target/boot" ] && [ "$mpoint" != "/target" ] && [ "$mpoint" != "/" ]; then
			type=$(grep "^$mapped " "$OS_PROBER_TMP/mounted-map" | head -n1 | cut -d " " -f 3)
			for test in /usr/lib/os-probes/mounted/*; do
				if [ -f "$test" ] && [ -x "$test" ]; then
					debug "running $test on mounted $partition"
					if "$test" "$partition" "$mpoint" "$type"; then
						debug "os detected by $test"
						break
					fi
				fi
			done
		fi
	fi
done
